#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <BlockNot.h>

#define WIFI_RETRIES    30

const IPAddress espIPAddress(10, 10, 11, 10);
const IPAddress subnetMask(255, 255, 255, 0);
const IPAddress defaultGateway(10, 10, 11, 1);
const IPAddress workstationIPAddress(10, 10, 10, 90);

const char *SSID = "ssid";
const char *wifiPassword = "password";
const uint16_t port = 7373;      // remote port to send on
String serialData = "";
bool stringComplete = false;

BlockNot pingTimer(5, SECONDS);

#define SEND_SERIAL Serial.println
#define PROCESS_UDP processUDP()
#define SEND_UDP sendUDP
#define PROCESS_SERIAL processSerial()

WiFiUDP UDP;

String getIPAddress(IPAddress ipAddress) {
    String ipaddy = String(ipAddress[0]) + ".";
    ipaddy += String(ipAddress[1]) + ".";
    ipaddy += String(ipAddress[2]) + ".";
    ipaddy += String(ipAddress[3]);
    return ipaddy;
}

void processUDP() {
    static int bufferSize = 500;
    char packetBuffer[bufferSize]; //buffer to hold incoming packet,
    int packetSize = UDP.parsePacket();
    if (packetSize) {
        int n = UDP.read(packetBuffer, packetSize);
        if (n > 2) {
            SEND_SERIAL(packetBuffer);
        }
    }
}

void sendUDP(String msg) {
    int packetSize = msg.length() + 1;
    char packet[packetSize];
    msg.toCharArray(packet, packetSize);
    UDP.beginPacket(workstationIPAddress, port);
    UDP.write(packet);
    UDP.endPacket();
}

void checkSerial() {
    while (Serial.available()) {
        char inChar = (char) Serial.read();
        serialData += inChar;
        if (inChar == '\n') {
            stringComplete = true;
        }
    }
}

void processSerial() {
    checkSerial();
    if (stringComplete) {
        SEND_UDP(serialData);
        serialData = "";
        stringComplete = false;
    }
}

void startWiFi() {
    int conn_tries = 0;
    WiFi.forceSleepWake();
    WiFi.mode(WIFI_STA);
    WiFi.config(espIPAddress, defaultGateway, subnetMask);
    WiFi.begin(SSID, wifiPassword);
    Serial.print("\nConnecting.");
    while ((WiFi.status() != WL_CONNECTED) && (conn_tries++ < WIFI_RETRIES)) {
        Serial.print(".");
        delay(200);
    }
    if (WiFi.status() == WL_CONNECTED) {
        UDP.begin(port);
        Serial.println("\nConnected!");
        Serial.println("localIP: " + getIPAddress(WiFi.localIP()));
        Serial.println("Subnet: " + getIPAddress(WiFi.subnetMask()));
        Serial.println("Gateway: " + getIPAddress(WiFi.gatewayIP()));
    }
}

void setup() {
    Serial.begin(57600);
    delay(500);
    startWiFi();
    yield();
    SEND_UDP("WiFi Repeater Ready!");
}

void loop() {
    PROCESS_UDP;
    PROCESS_SERIAL;
    if (pingTimer.TRIGGERED) {
        SEND_UDP("pingwifi");
    }
}
